/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_COMMON_FMATRIXCOL_HH
#define DUNE_FEM_COMMON_FMATRIXCOL_HH

#include <type_traits>

#include <dune/common/densevector.hh>
#include <dune/common/fmatrix.hh>

namespace Dune
{

  // Internal Forward Declarations
  // -----------------------------

  template< class FieldMatrix >
  class FieldMatrixColumn;



  // DenseMatVecTraits for FieldMatrixColumn
  // ---------------------------------------

  template< class FieldMatrix >
  struct DenseMatVecTraits< FieldMatrixColumn< FieldMatrix > >
  {
    typedef FieldMatrixColumn< FieldMatrix > derived_type;

    typedef typename DenseMatVecTraits< typename std::remove_const< FieldMatrix >::type >::value_type value_type;
    typedef typename DenseMatVecTraits< typename std::remove_const< FieldMatrix >::type >::size_type size_type;
  };



  // FieldMatrixColumn
  // -----------------

  template< class K, int m, int n >
  class FieldMatrixColumn< FieldMatrix< K, m, n > >
    : public DenseVector< FieldMatrixColumn< FieldMatrix< K, m, n > > >
  {
    typedef DenseVector< FieldMatrixColumn< FieldMatrix< K, m, n > > > Base;

  public:
    static const int dimension = m;

    typedef typename Base::size_type size_type;
    typedef typename Base::value_type value_type;

    FieldMatrixColumn ( FieldMatrix< K, m, n > &fieldMatrix, int column )
      : fieldMatrix_( fieldMatrix ),
        column_( column )
    {}

    using Base::operator=;

    DUNE_CONSTEXPR size_type size () const { return vec_size(); }

    DUNE_CONSTEXPR size_type vec_size () const { return dimension; }
    const value_type &vec_access ( size_type i ) const { return fieldMatrix_[ i ][ column_ ]; }
    value_type &vec_access ( size_type i ) { return fieldMatrix_[ i ][ column_ ]; }

  private:
    FieldMatrix< K, m, n > &fieldMatrix_;
    int column_;
  };

  template< class K, int m, int n >
  class FieldMatrixColumn< const FieldMatrix< K, m, n > >
    : public DenseVector< FieldMatrixColumn< const FieldMatrix< K, m, n > > >
  {
    typedef DenseVector< FieldMatrixColumn< const FieldMatrix< K, m, n > > > Base;

  public:
    static const int dimension = m;

    typedef typename Base::size_type size_type;
    typedef typename Base::value_type value_type;

    FieldMatrixColumn ( const FieldMatrix< K, m, n > &fieldMatrix, int column )
      : fieldMatrix_( fieldMatrix ),
        column_( column )
    {}

    using Base::operator=;

    DUNE_CONSTEXPR size_type size () const { return vec_size(); }

    DUNE_CONSTEXPR size_type vec_size () const { return dimension; }
    const value_type &vec_access ( size_type i ) const { return fieldMatrix_[ i ][ column_ ]; }

  private:
    const FieldMatrix< K, m, n > &fieldMatrix_;
    int column_;
  };

} // namespace Dune

#endif // #ifndef DUNE_FEM_COMMON_FMATRIXCOL_HH
