/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_COMMON_TYPEINDEXEDTUPLE_HH
#define DUNE_FEM_COMMON_TYPEINDEXEDTUPLE_HH

#include <dune/common/tuples.hh>
#include <dune/common/tupleutility.hh>

#include <dune/fem/common/tupleutility.hh>

namespace Dune
{

  // TypeIndexedTuple
  // ----------------

  /*
   * \brief Please doc me.
   */
  template< class Tuple, class Types >
  class TypeIndexedTuple
  {
    template< class T >
    struct Position
    {
      static const int value = Dune::FirstTypeIndex< Types, T >::value;
    };

    // value selector for types that are contained
    template< class T, bool contained >
    struct ValueBase
    {
      typedef typename std::tuple_element< Position< T >::value, Tuple >::type Type;
      static Type& at( Tuple& tuple )
      {
        return Dune::get< Position< T >::value >( tuple );
      }
      static const Type& at( const Tuple& tuple )
      {
        return Dune::get< Position< T >::value >( tuple );
      }
    };

    // value selector for types that are not contained
    template< class T >
    struct ValueBase< T, false > // T is not contained in Tuple
    {
      typedef Tuple Type;
      static Type& at( Tuple& tuple )
      {
        return tuple;
      }
      static const Type& at( const Tuple& tuple )
      {
        return tuple;
      }
    };

  public:
    template <class T>
    struct Contains
    {
      static const bool value = ContainsType< Types, T >::value;
    };

    template< class T >
    struct Value : public ValueBase< T, Contains< T >::value >
    {
    };

    explicit TypeIndexedTuple ( const Tuple &tuple = Tuple() )
    : tuple_( tuple )
    {}

    //! \brief return reference to tuple member associated with type T
    template< class T >
    typename Value< T >::Type &at ()
    {
      return Value< T >::at( tuple_ );
    }

    //! \brief return reference to tuple member associated with type T
    template< class T >
    const typename Value< T >::Type &at () const
    {
      return Value< T >::at( tuple_ );
    }

    //! \brief return reference to tuple member associated with type T (integral_constant)
    template< class T >
    typename Value< T >::Type &operator[] ( const T & )
    {
      return at< T >();
    }

    //! \brief return reference to tuple member associated with type T (integral_constant)
    template< class T >
    const typename Value< T >::Type &operator[] ( const T & ) const
    {
      return at< T >();
    }

    //! \brief return true if type T is contained in the tuple
    template< class T >
    const bool active( const T& ) const { return Contains< T >::value; }

    //! \brief cast to Tuple
    operator Tuple & () { return tuple_; }

    //! \brief cast to const Tuple
    operator const Tuple & () const { return tuple_; }

  private:
    Tuple tuple_;
  };



  // get for TypeIndexedTuple
  // ------------------------

  template< int i, class Tuple, class Types >
  typename tuple_element< i, Tuple >::type &
  get ( Dune::TypeIndexedTuple< Tuple, Types > &tuple )
  {
    return get< i >( static_cast< Tuple & >( tuple ) );
  }

  template< int i, class Tuple, class Types >
  const typename tuple_element< i, Tuple >::type &
  get ( const Dune::TypeIndexedTuple< Tuple, Types > &tuple )
  {
    return get< i >( static_cast< const Tuple & >( tuple ) );
  }

} // namespace Dune



// Some Specializations for Tuple Access
// -------------------------------------

DUNE_OPEN_TUPLE_NAMESPACE

  // tuple_element for TypeIndexedTuple
  // ----------------------------------

  template< size_t i, class Tuple, class Types >
  struct tuple_element< i, Dune::TypeIndexedTuple< Tuple, Types > >
  {
    typedef typename tuple_element< i, Tuple >::type type;
  };

DUNE_CLOSE_TUPLE_NAMESPACE

#endif // #ifndef DUNE_FEM_COMMON_TYPEINDEXEDTUPLE_HH
