/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_CAPABILITIES_HH
#define DUNE_FEM_GRIDPART_CAPABILITIES_HH


namespace Dune
{

  namespace Fem
  {

    namespace GridPartCapabilities
    {
      /** \brief specialize with 'false' if grid part has no
       *         underlying dune grid (default=true)
       */
      template< class GridPartType >
      struct hasGrid
      {
        static const bool v = true;
      };


      /** \brief specialize with 'true' for if the codimension 0 entity
       *         of the grid part has only one possible geometry type
       *         (default=false, topologyid=undefined)
       */
      template< class GridPartType >
      struct hasSingleGeometryType
      {
        static const bool v = false;
        static const unsigned int topologyId = ~0u;
      };


      /** \brief specialize with 'true' if the grid part is
       *         cartesian (default=false)
       */
      template< class GridPartType >
      struct isCartesian
      {
        static const bool v = false;
      };


      /** \brief specialize with 'true' for all codims that a
                 grid implements entities for (default=false)
      */
      template< class GridPartType, int codim  >
      struct hasEntity
      {
        static const bool v = false;
      };


      /** \brief specialize with 'true' if implementation supports
       *         parallelism (default=false)
       */
      template< class GridPartType >
      struct isParallel
      {
        static const bool v = false;
      };


      /** \brief specialize with 'true' for all codims that a
       *         grid can communicate data on (default=false)
       */
      template< class GridPartType, int codim >
      struct canCommunicate
      {
        static const bool v = false;
      };


      /** \brief specialize with 'true' if implementation guarantees
       *         conforming level grids. (default=false)
       */
      template< class GridPartType >
      struct isConforming
      {
        static const bool v = false;
      };


      /*
       * forward
       *   GridPartCapabilities::Something< const GridPartType >
       * to
       *   GridPartCapabilities::Something< GridPartType >
       */

      template< class GridPartType >
      struct hasGrid< const GridPartType >
      {
        static const bool v = Dune::Fem::GridPartCapabilities::hasGrid< GridPartType >::v;
      };


      template< class GridPartType >
      struct hasSingleGeometryType< const GridPartType >
      {
        static const bool v = Dune::Fem::GridPartCapabilities::hasSingleGeometryType< GridPartType >::v;
        static const unsigned int topologyId
          = Dune::Fem::GridPartCapabilities::hasSingleGeometryType< GridPartType >::topologyId;
      };


      template< class GridPartType >
      struct isCartesian< const GridPartType >
      {
        static const bool v = Dune::Fem::GridPartCapabilities::isCartesian< GridPartType >::v;
      };


      template< class GridPartType, int codim  >
      struct hasEntity< const GridPartType, codim >
      {
        static const bool v = Dune::Fem::GridPartCapabilities::hasEntity< GridPartType, codim >::v;
      };


      template< class GridPartType >
      struct isParallel< const GridPartType >
      {
        static const bool v = Dune::Fem::GridPartCapabilities::isParallel< GridPartType >::v;
      };


      template< class GridPartType, int codim >
      struct canCommunicate< const GridPartType, codim >
      {
        static const bool v = Dune::Fem::GridPartCapabilities::canCommunicate< GridPartType, codim >::v;
      };


      template< class GridPartType >
      struct isConforming< const GridPartType >
      {
        static const bool v = Dune::Fem::GridPartCapabilities::isConforming< GridPartType >::v;
      };

    } // namespace GridPartCapabilities

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_CAPABILITIES_HH
