/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_COMMON_DEFAULTGRIDPARTENTITY_HH
#define DUNE_FEM_GRIDPART_COMMON_DEFAULTGRIDPARTENTITY_HH

#include <type_traits>

#include <dune/common/exceptions.hh>

#include <dune/grid/common/entity.hh>
#include <dune/grid/common/gridenums.hh>

namespace Dune
{

  namespace Fem
  {

    // DefaultGridPartEntity
    // ---------------------

    template< int codim, int dim, class GridFamily >
    class DefaultGridPartEntity
    {
    protected:
      typedef typename std::remove_const< GridFamily >::type::Traits Traits;

    public:
      static const int codimension = codim;
      static const int dimension = dim;
      static const int mydimension = dimension - codimension;

      int level () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }
    };



    // DefaultGridPartEntity for codimension 0
    // ---------------------------------------

    template< int dim, class GridFamily >
    class DefaultGridPartEntity< 0, dim, GridFamily >
    {
    protected:
      typedef typename std::remove_const< GridFamily >::type::Traits Traits;

    public:
      static const int codimension = 0;
      static const int dimension = dim;
      static const int mydimension = dimension - codimension;

      typedef typename Traits::template Codim< codimension >::LocalGeometry LocalGeometry;
      typedef typename Traits::template Codim< codimension >::Entity Entity;

      typedef typename Traits::HierarchicIterator HierarchicIterator;

      int level () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      bool isLeaf () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      Entity father () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      bool hasFather () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      const LocalGeometry &geometryInFather () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      HierarchicIterator hbegin ( int maxLevel ) const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      HierarchicIterator hend ( int maxLevel ) const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      bool isRegular () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      bool isNew () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }

      bool mightVanish () const
      {
        DUNE_THROW( InvalidStateException, "Trying to access hierarchy information from a grid part." );
      }
    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_COMMON_DEFAULTGRIDPARTENTITY_HH
