/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_COMMON_POLICIES_HH
#define DUNE_FEM_GRIDPART_COMMON_POLICIES_HH

#include <type_traits>
#include <utility>

#include <dune/grid/common/gridview.hh>

#include <dune/fem/gridpart/common/gridpart2gridview.hh>

namespace Dune
{

  namespace Fem
  {

    // Internal forward declaration
    // ----------------------------

    template< class Traits >
    class GridPartPolicies;



#ifndef DOXYGEN

    namespace __GridPartPolicies
    {

      // hasGridView
      // -----------

      template< class Traits >
      std::true_type __hasGridView ( const Traits &, const typename Traits::GridViewType * = nullptr );

      std::false_type __hasGridView ( ... );

      template< class Traits >
      struct hasGridView
      {
        static const bool v = decltype( __hasGridView( std::declval< Traits >() ) )::value;
      };



      // HasGridView
      // -----------

      template< class Traits, bool hasGridView = hasGridView< Traits >::v >
      class HasGridView;

      template< class Traits >
      class HasGridView< Traits, true >
      {
      public:
        typedef typename Traits::GridViewType GridViewType;
      };

      template< class Traits >
      class HasGridView< Traits, false >
      {
        typedef typename Traits::GridPartType GridPartType;

      public:
        typedef Dune::GridView< GridPart2GridViewTraits< GridPartType > > GridViewType;

      protected:
        HasGridView () = default;

      public:
        operator GridPart2GridViewImpl< GridPartType > () const
        {
          return GridPart2GridViewImpl< GridPartType >( impl() );
        }

      private:
        const GridPartType &impl () const
        {
          return static_cast< const GridPartType & >( *this );
        }
      };

    } // namespace __GridPartPolicies

#endif // #ifndef DOXYGEN



    // GridPartPolicies
    // ----------------

    template< class Traits >
    class GridPartPolicies
      : public __GridPartPolicies::HasGridView< Traits >
    {};

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_COMMON_POLICIES_HH
