/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_FILTER_INVERSEFILTER_HH
#define DUNE_FEM_GRIDPART_FILTER_INVERSEFILTER_HH

namespace Dune
{

  namespace Fem
  {

    template< class FilterImp >
    class InverseFilter
    {
    public:
      //! \brief type of the filter implementation
      typedef InverseFilter< FilterImp > FilterType;

      //! \brief entity types
      template< int cd >
      struct Codim
      {
        typedef typename FilterImp::template Codim< cd >::EntityType EntityType;
      };

      //! \brief type of entity with codim=0
      typedef typename Codim< 0 >::EntityType EntityType;

      //! \brief constructor
      InverseFilter ( const FilterImp & filter = FilterImp() )
      : filter_( filter )
      { }

      //! \brief returns true if the given entity of the pointer in the domain
      template< int cd >
      bool contains ( const typename Codim< cd >::EntityType & entity ) const
      {
        return !filter().contains< cd >( entity );
      }

      //! \brief returns true if the given entity of the pointer in the domain
      template< class Entity >
      bool contains ( const Entity & entity ) const
      {
        return !filter().contains( entity );
      }

      //! \brief returns true if an intersection is interior
      //         (allows boundarys within a given domain)
      template< class Intersection >
      bool interiorIntersection ( const Intersection &intersection ) const
      {
        typedef typename Intersection::Entity EntityType;
        const EntityType outside(intersection.outside());
        return contains( outside );
      }

      //! \brief returns true if an intersection is a boundary intersection
      template< class Intersection >
      bool intersectionBoundary( const Intersection &intersection ) const
      {
        return filter().intersectionBoundary( intersection );
      }

      //! \brief returns the boundary id for an intersection
      template< class Intersection >
      int intersectionBoundaryId ( const Intersection &intersection ) const
      {
        return filter().intersectionBoundaryId( intersection );
      }

      //! \brief returns true if for an intersection a neighbor exsits
      template< class Intersection >
      bool intersectionNeighbor ( const Intersection &intersection ) const
      {
        return filter().intersectionNeighbor( intersection );
      }

    private:
      const FilterImp filter () const
      {
        return filter_;
      }

      FilterImp filter_;

    }; // end class InverseFilter

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_FILTER_INVERSEFILTER_HH
