/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_GEOGRIDPART_CAPABILITIES_HH
#define DUNE_FEM_GRIDPART_GEOGRIDPART_CAPABILITIES_HH

//- dune-fem includes
#include <dune/fem/gridpart/common/capabilities.hh>

namespace Dune
{

  namespace Fem
  {

    // Forward declaration
    // -------------------

    template< class >
    class GeoGridPart;



    namespace GridPartCapabilities
    {

      template< class CoordFunction >
      struct hasGrid< GeoGridPart< CoordFunction > >
      {
        // either implement this or leaf it away !!!
//        static const bool v = hasGrid< typename CoordFunction::GridPartType >::v;
        static const bool v = false;
      };


      template< class CoordFunction >
      class hasSingleGeometryType< GeoGridPart< CoordFunction > >
      {
        typedef typename CoordFunction::GridPartType HostGridPartType;

      public:
        static const bool v = hasSingleGeometryType< HostGridPartType >::v;
        static const unsigned int topologyId = hasSingleGeometryType< HostGridPartType >::topologyId;
      };


      template< class CoordFunction >
      struct isCartesian< GeoGridPart< CoordFunction > >
      {
        static const bool v = false;
      };


      template< class CoordFunction, int codim  >
      struct hasEntity< GeoGridPart< CoordFunction >, codim >
      {
        // disable codim > 0 && < dim entities because of missing interface for subIndex method
        // once this is implemented we can simply use hasEntity for the HostGridPart.
        static const bool v = ( codim == 0 || codim == CoordFunction::GridPartType :: dimension ) ?
            hasEntity< typename CoordFunction::GridPartType, codim >::v : false ;
      };


      template< class CoordFunction >
      struct isParallel< GeoGridPart< CoordFunction > >
      {
        static const bool v = isParallel< typename CoordFunction::GridPartType >::v;
      };


      template< class CoordFunction, int codim >
      struct canCommunicate< GeoGridPart< CoordFunction >, codim >
      {
        static const bool v = canCommunicate< typename CoordFunction::GridPartType, codim >::v;
      };


      template< class CoordFunction >
      struct isConforming< GeoGridPart< CoordFunction > >
      {
        static const bool v = isConforming< typename CoordFunction::GridPartType >::v;
      };

    } // namespace GridPartCapabilities

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_GEOGRIDPART_CAPABILITIES_HH
