/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_GRIDPART_TEST_FAILURE_HH
#define DUNE_FEM_GRIDPART_TEST_FAILURE_HH

#include <cassert>

//- dune-common includes
#include <dune/common/stdstreams.hh>

//- include this as very last
#include <dune/common/bartonnackmanifcheck.hh>


namespace Dune
{

  // Failure
  // -------

  struct Failure
  {
    /** \brief default constructor  */
    Failure () { }

  protected:
    friend std::ostream& operator<< ( std::ostream &, const Failure & );

    /** \brief print output to ostream */
    virtual void writeTo ( std::ostream &out ) const = 0;
  };

  std::ostream &operator<< ( std::ostream &s, const Failure &failure )
  {
    failure.writeTo( s );
    return s;
  }


  // FailureHandler
  // --------------

  template< class FailureHandlerImp >
  struct FailureHandler
  {
    /** \brief default constructor  */
    FailureHandler () { }

    /** \brief act on failure */
    template< class Failure >
    void operator() ( Failure &failure ) const
    {
      CHECK_INTERFACE_IMPLEMENTATION( ( asImp().operator()( failure ) ) );
      return asImp().operator()( failure );
    }

  private:
    // hide copy constructor
    FailureHandler ( const FailureHandler & );
    // hide assignment operator
    FailureHandler &operator= ( const FailureHandler & );

    //  Barton-Nackman trick
    FailureHandlerImp &asImp ()
    {
      return static_cast< FailureHandlerImp & >( *this );
    }

    //  Barton-Nackman trick
    const FailureHandlerImp &asImp () const
    {
      return static_cast< const FailureHandlerImp & >( *this );
    }
  };



  // DefaultFailureHandler
  // ---------------------

  struct DefaultFailureHandler
  : public FailureHandler< DefaultFailureHandler >
  {
    /** \brief act on failure */
    template< class Failure >
    void operator() ( Failure &failure ) const
    {
      std::cerr<< failure << std::endl;
      assert( false );
    }
  };

} // end namespace Dune

#endif // #ifndef DUNE_FEM_GRIDPART_TEST_FAILURE_HH
