/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_ASCIIPARSER_HH
#define DUNE_FEM_ASCIIPARSER_HH

#include <iostream>
#include <fstream>
#include <string>

namespace Dune
{

  namespace Fem
  {

    //! reads data folowing the given keyword
    //! if verbose is true then an output of what was read is given
    //! the token '%' or '#' stands for comment
    template< class T >
    static inline bool readParameter( std::istream& file,
                        const std::string keyword,
                        T & data,
                        bool verbose = true,
                        bool warn = true )
    {
      bool readData = false;
      while (! file.eof() )
      {
        std::string  keyHelp;
        file >> keyHelp;

        // % or # means comment
        if((keyHelp[0] == '%') || (keyHelp[0] == '#'))
        {
          std::string tmp;
          std::getline(file,tmp);
        }
        else
        {
          // copy only keyword size and compare
          int pos = keyHelp.find_first_of(':');
          int pos1 = keyHelp.find_first_of(' ');

          if (pos > 0)
          {
            if(pos1 > 0)
              pos -=  pos1;
            else
              pos = 1;
          }
          else
            pos = 0;

          std::string key (keyHelp,0,keyHelp.size() - pos);
          if(key == keyword)
          {
            file >> data;
            readData = true;
            break;
          }
        }
      }

      // if data was read sucessfully
      if(readData)
      {
        if(verbose)
        {
          static const int MAXTAB = 30;
          int length = MAXTAB - keyword.size();
          std::cout << "Reading " << keyword;
          for(int i=0; i<length; i++) std::cout << ".";
          std::cout << " " << data << std::endl;
        }
      }
      else
      {
        if( warn )
          std::cerr << "WARNING: couldn't read " << keyword << std::endl;
      }

      return readData;
    }

    //! reads data folowing the given keyword
    //! if verbose is true then an output of what was read is given
    //! the token '%' or '#' stands for comment
    template< class T >
    static inline bool readParameter( const std::string filename,
                        const std::string keyword,
                        T & data,
                        bool verbose = true,
                        bool warn = true )
    {
      std::ifstream file (filename.c_str());
      if( !file.is_open() )
      {
        if( warn )
          std::cerr << "WARNING: couldn't open file '" << filename << "' in " <<  __FILE__<< " line: " << __LINE__ << std::endl;
        return false;
      }
      const bool result = readParameter( file, keyword, data, verbose, warn );

      // close file
      file.close();

      return result;
    }

  } // namespace Fem

} // namespace  Dune
#endif // #ifndef DUNE_FEM_ASCIIPARSER_HH
