/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_IO_PARAMETER_PARSER_HH
#define DUNE_FEM_IO_PARAMETER_PARSER_HH

#include <sstream>
#include <string>
#include <type_traits>

#include <dune/common/fmatrix.hh>

namespace Dune
{

  namespace Fem
  {

    // ParameterParser
    // ---------------

    template< class T >
    struct ParameterParser
    {
      static bool parse ( const std::string &s, T &value )
      {
        std::istringstream in( s );
        in >> value;
        if( std::is_same< T, std::string >::value && s.empty() )
          return true;
        if( in.fail() )
          return false;
        char eof;
        in >> eof;
        return in.eof();
      }

      static std::string toString ( const T &value )
      {
        std::ostringstream out;
        out << value;
        return out.str();
      }
    };

    template<>
    struct ParameterParser< bool >
    {
      static bool parse ( const std::string &s, bool &value )
      {
        std::string w;
        if( ParameterParser< std::string >::parse( s, w ) )
        {
          if( (w == std::string( "false" )) || (w == std::string( "no" )) || (w == std::string( "0" )) )
          {
            value = false;
            return true;
          }

          if( (w == std::string( "true" )) || (w == std::string( "yes" )) || (w == std::string( "1" )) )
          {
            value = true;
            return true;
          }
        }
        return false;
      }

      static std::string toString ( const bool &value )
      {
        return std::string( value ? "true" : "false" );
      }
    };

    template< class F, int m, int n >
    struct ParameterParser< FieldMatrix< F, m, n > >
    {
      static bool parse ( const std::string &s, FieldMatrix< F, m, n > &value )
      {
        std::istringstream in( s );
        char c;
        for( int i = 0; i < m; ++i )
        {
          if( i > 0 )
          {
            in >> c;
            if( c != ',' )
              return false;
          }

          for( int j = 0; j < n; ++j )
            in >> value[ i ][ j ];
        }
        in >> c; // read eof
        return in.eof();
      }

      static std::string toString ( const FieldMatrix< F, m, n > &value )
      {
        std::ostringstream out;
        for( int i = 0; i < m; ++i )
        {
          out << (i > 0 ? "," : "");
          for( int j = 0; j< n; ++j )
            out << " " << value[ i ][ j ];
        }
        return out.str();
      }
    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_IO_PARAMETER_PARSER_HH
