/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_BINARYSTREAMS_HH
#define DUNE_FEM_BINARYSTREAMS_HH

#include <dune/fem/io/streams/standardstreams.hh>

namespace Dune
{

  namespace Fem
  {

    /** \class BinaryFileOutStream
     *  \copydoc StandardOutStream
     */
    class BinaryFileOutStream : public StandardOutStream
    {
      typedef BinaryFileOutStream ThisType;
      typedef StandardOutStream   BaseType;

      // make copy constructor private because of the file pointers
      BinaryFileOutStream( const BinaryFileOutStream& );
    public:
      /** \brief constructor
       *
       *  \param[in]  filename  name of a file to write to
       */
      explicit BinaryFileOutStream ( const std::string &filename )
        : BaseType( openFile( filename ) )
      {
      }

      /** \brief destructor deleteing file stream */
      ~BinaryFileOutStream()
      {
        delete file_; file_ = 0;
      }
    protected:
      std::ostream& openFile( const std::string& filename )
      {
        // init file
        file_ = new std::ofstream( filename.c_str(), std::ios::binary );

        if( ! (*file_) )
          DUNE_THROW( Dune::IOError, "Unable to open file: " << filename );

        return *file_;
      }

      //! standard file stream
      std::ofstream* file_;
    };

    /** \class BinaryFileInStream
     *  \copydoc StandardInStream
     */
    class BinaryFileInStream: public StandardInStream
    {
      typedef BinaryFileInStream ThisType;
      typedef StandardInStream   BaseType;

      // make copy constructor private because of the pointers
      BinaryFileInStream ( const BinaryFileInStream& );

    public:
      /** \brief constructor
       *
       *  \param[in]  filename  name of a file to write to
       */
      explicit BinaryFileInStream ( const std::string &filename )
      : BaseType( openFile( filename ) )
      {
      }

      /** \brief destructor deleteing file stream */
      ~BinaryFileInStream()
      {
        delete file_; file_ = 0;
      }

    protected:
      std::istream& openFile( const std::string& filename )
      {
        file_ = (new std::ifstream( filename.c_str(), std::ios::binary ));

        if( ! (*file_) )
          DUNE_THROW( Dune::IOError, "Unable to open file: " << filename );

        return *file_;
      }

      //! standard file stream
      std::ifstream* file_;
    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_BINARYSTREAMS_HH
