/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_IO_STREAMS_TUPLES_HH
#define DUNE_FEM_IO_STREAMS_TUPLES_HH

#include <dune/common/tuples.hh>
#include <dune/common/tupleutility.hh>

#include <dune/fem/io/streams/streams.hh>

namespace
{

  // TupleToInStream
  // ---------------

  template< class InStream >
  class TupleToInStream
  {
    struct InStreamFunctor
    {
      InStreamFunctor ( InStream &in ) : in_( in ) {}

      template< class T >
      void visit ( T &t ) const
      {
        in_ >> t;
      }

      private:
        InStream &in_;
    };

  public:
    template< class Tuple >
    static InStream &apply ( InStream &in, Tuple &tuple )
    {
      Dune::ForEachValue< Tuple > forEach( tuple );
      InStreamFunctor functor( in );
      forEach.apply( functor );
      return in;
    }
  };



  // TupleToOutStream
  // ----------------

  template< class OutStream >
  class TupleToOutStream
  {
    struct OutStreamFunctor
    {
      OutStreamFunctor ( OutStream &out ) : out_( out ) {}

      template< class T >
      void visit ( T &t ) const
      {
        out_ << t;
      }

      private:
        OutStream &out_;
    };

  public:
    template< class Tuple >
    static OutStream &apply ( OutStream &out, const Tuple &tuple )
    {
      Dune::ForEachValue< Tuple > forEach( const_cast< Tuple & >( tuple ) );
      OutStreamFunctor functor( out );
      forEach.apply( functor );
      return out;
    }
  };

} // namespace



namespace Dune
{

  namespace Fem
  {

    // External forward declarations
    // -----------------------------

    template <class StreamTraits>
    class OutStreamInterface;
    template <class StreamTraits>
    class InStreamInterface;



    // std::tuple to InStream
    // ----------------------

    template< class StreamTraits >
    inline InStreamInterface< StreamTraits > &
      operator>> ( InStreamInterface< StreamTraits > &in, std::tuple<> &tuple )
    {
      return in;
    }

    template< class StreamTraits, class... Args >
    inline InStreamInterface< StreamTraits > &
      operator>> ( InStreamInterface< StreamTraits > &in, std::tuple< Args... > &tuple )
    {
      return TupleToInStream< InStreamInterface< StreamTraits > >::template apply< std::tuple< Args... > >( in, tuple );
    }



    // std::tuple to OutStream
    // -----------------------

    template< class StreamTraits >
    inline OutStreamInterface< StreamTraits > &
      operator<< ( OutStreamInterface< StreamTraits > &out, const std::tuple<> &tuple )
    {
      return out;
    }

    template< class StreamTraits, class... Args >
    inline OutStreamInterface< StreamTraits > &
      operator<< ( OutStreamInterface< StreamTraits > &out, const std::tuple< Args... > &tuple )
    {
      return TupleToOutStream< OutStreamInterface< StreamTraits > >::template apply< std::tuple < Args... > >( out, tuple );
    }

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_IO_STREAMS_TUPLES_HH
