/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
//************************************************************
//
//  (C) written and directed by Robert Kloefkorn
//
//************************************************************
#include <iostream>
#include <vector>
#include <cassert>
#include <string>

#if HAVE_MPI == 1
#warning "Visualization does not work in parallel"
#endif

#include <dune/common/exceptions.hh>
using namespace Dune;

// include definition of grid type
// #include <dune/grid/io/file/dgfparser/dgfgridtype.hh>

// include data reading
#include <dune/fem/io/visual/grape/datadisp/printhelp.cc>

// uses readtuple data instead of readiotupledata.
#include <dune/fem/io/visual/grape/datadisp/readtupledata.cc>
#include <dune/fem/io/visual/grape/datadisp/readioparams.cc>
#include <dune/fem/misc/mpimanager.hh>

int main(int argc, char **argv)
{
  Fem::MPIManager::initialize(argc,argv);
  try {
    Fem::Parameter::append(argc,argv);
#ifdef PARAMETER_APPEND_FUNCTION
    PARAMETER_APPEND_FUNCTION();
#endif

    if (argc < 2)
    {
      print_help(argv[0]);
      return(0);
    }

    if (!strcmp(argv[1], "-h") || !strcmp(argv[1], "-help"))
    {
      print_help(argv[0]);
      return(0);
    }
    return readParameterList(argc,argv,false);
  }
  catch (Dune::Exception& e)
  {
    std::cerr << e << std::endl;
    return 1;
  }
  return 0;
}
