/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_FIELDMATRIXHELPER_HH
#define DUNE_FEM_FIELDMATRIXHELPER_HH

#include <dune/common/fvector.hh>
#include <dune/common/fmatrix.hh>
#include <dune/common/dynmatrix.hh>

namespace Dune
{

  namespace Fem
  {

    namespace FieldMatrixHelper
    {

      template< class Field1, class Field2, class Field3, int m, int n >
      inline void multiply ( const FieldMatrix< Field1, m, n > &A,
                             const FieldVector< Field2, n > &x,
                             FieldVector< Field3, m > &y )
      {
        for( int i = 0; i < m; ++i )
        {
          Field3 &value = y[ i ];

          value = 0;
          for( int j = 0; j < n; ++j )
            value += A[ i ][ j ] * x[ j ];
        }
      }



      template< class Field1, class Field2, class Field3, int m, int n, int p >
      inline void multiply ( const FieldMatrix< Field1, m, n > &A,
                             const FieldMatrix< Field2, n, p > &B,
                             FieldMatrix< Field3, m, p > &C )
      {
        for( int i = 0; i < m; ++i )
        {
          for( int j = 0; j < p; ++j )
          {
            Field3 &value = C[ i ][ j ];

            value = 0;
            for( int k = 0; k < n; ++k )
              value += A[ i ][ k ] * B[ k ][ j ];
          }
        }
      }

      template< class Field1, class Field2, class Field3 >
      inline void multiply ( const DynamicMatrix< Field1 >& A,
                             const DynamicMatrix< Field2 >& B,
                             DynamicMatrix< Field3 >& C )
      {
        const int m = A.rows();
        const int n = A.cols();
        const int p = B.cols();

        C.resize( m, p );

        assert( A.cols() == B.rows() );
        assert( A.rows() == C.rows() );
        assert( B.cols() == C.cols() );

        for( int i = 0; i < m; ++i )
        {
          for( int j = 0; j < p; ++j )
          {
            Field3 &value = C[ i ][ j ];

            value = 0;
            for( int k = 0; k < n; ++k )
              value += A[ i ][ k ] * B[ k ][ j ];
          }
        }
      }

      template< class Field1, class Field2, class Field3, int m, int n, int p >
      inline void multiply ( const FieldMatrix< Field1, m, n > &A,
                             const FieldMatrix< Field2, n, p > &B,
                             Field3* C)
      {
        for( int i = 0, ip = 0; i < m; ++i )
        {
          for( int j = 0; j < p; ++j , ++ ip )
          {
            Field3 &value = C[ ip ];
            value = 0;
            for( int k = 0; k < n; ++k )
              value += A[ i ][ k ] * B[ k ][ j ];
          }
        }
      }

    }

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_FIELDMATRIXHELPER_HH
