/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
// vim: set expandtab ts=2 sw=2 sts=2:
#ifndef DUNE_FEM_PETSCSLAVEDOFPROVIDER_HH
#define DUNE_FEM_PETSCSLAVEDOFPROVIDER_HH

#include <dune/fem/function/common/scalarproducts.hh>


#if HAVE_PETSC

#include <dune/fem/misc/petsc/petsccommon.hh>
#include <dune/fem/misc/petsc/petscdofmappings.hh>


namespace Dune
{

  namespace Fem
  {

    /* =================================================
     * class PetscSlaveDofProvider
     * =================================================
     */
    template< typename DFSpace >
    class PetscSlaveDofProvider : public SlaveDofsProvider< DFSpace >
    {
    public:
      typedef DFSpace DiscreteFunctionSpaceType;

      typedef PetscSlaveDofProvider< DiscreteFunctionSpaceType > ThisType;
      typedef SlaveDofsProvider< DiscreteFunctionSpaceType >     BaseType;
    protected:
      using BaseType :: space_;
      using BaseType :: slaveDofs_;

    public:
      typedef typename DiscreteFunctionSpaceType :: BlockMapperType  BlockMapperType;
      typedef SlaveDofs< DiscreteFunctionSpaceType, BlockMapperType > SlaveDofsType;


      // type of communication manager object which does communication
      typedef PetscDofMappings< SlaveDofsType >  PetscDofMappingType;
      typedef SingletonList< SlaveDofsType*, PetscDofMappingType > PetscDofMappingProviderType;

      explicit PetscSlaveDofProvider ( const DiscreteFunctionSpaceType &space )
      : BaseType( space ),
        dofMapping_( PetscDofMappingProviderType::getObject( slaveDofs_ ) )
      {
      }

      //! destructor
      ~PetscSlaveDofProvider()
      {
        // free mapping
        PetscDofMappingProviderType::removeObject( dofMapping_ );
      }

      const PetscDofMappingType& dofMapping() const { return dofMapping_; }
      PetscDofMappingType& dofMapping() { return dofMapping_; }

    protected:
      // the globally unique dof mapping
      PetscDofMappingType& dofMapping_;
    };


  } // namespace Fem

} // namespace Dune

#endif // #if HAVE_PETSC

#endif // DUNE_FEM_PETSCSLAVEDOFPROVIDER_HH
