/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_OBJPOINTER_HH
#define DUNE_FEM_OBJPOINTER_HH

namespace Dune
{

  namespace Fem
  {

    //! storage class for newly generated operators during operator + on
    //! DiscreteOperator
    template <class ObjType>
    struct ObjPointer
    {
      typedef ObjPointer<ObjType> ObjPointerType;
      //! remember object item and next pointer
      ObjType * item;
      ObjPointerType *next;

      //! new ObjPointer is only created with pointer for item
      ObjPointer () : item (0) , next (0) {}

      //! new ObjPointer is only created with pointer for item
      ObjPointer (ObjType  *t) : item (t) , next (0) {}

      //! delete the next ObjPointer and the item
      //! need virtual desctructor for deleting the real objects.
      virtual ~ObjPointer ()
      {
        if(next) delete next; next = 0;
        if(item) delete item; item = 0;
      }
    };

    //! stores the new created objects when DiscreteOperatorDefault
    //! operator + or operator * is called
    class ObjPointerStorage
    {
      typedef ObjPointerStorage MyType;
    public:
      //! make new operator with item points to null
      ObjPointerStorage () : item_ (0) {}

      //! need virtual desctructor for deleting the real objects.
      virtual ~ObjPointerStorage ()
      {
        if(item_) delete item_; item_ = 0;
      }

      //! Store new generated DiscreteOperator Pointer
      template <class DiscrOpType>
      void saveObjPointer ( DiscrOpType * discrOp )
      {
        ObjPointerType *next = new ObjPointerType ( discrOp );
        next->next = item_;
        item_ = next;
      }

      //! Store new generated DiscreteOperator Pointer and the LocalOperator
      //! pointer
      template <class DiscrOpType, class LocalOpType >
      void saveObjPointer ( DiscrOpType * discrOp , LocalOpType * lop )
      {
        saveObjPointer( discrOp );
        saveObjPointer( lop     );
      }

    private:
      // store the objects created by operator + in here
      typedef ObjPointer<MyType> ObjPointerType;
      ObjPointerType * item_;

    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_OBJPOINTER_HH
