/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#include <config.h>

#include <iostream>

#include <dune/common/fvector.hh>
#include <dune/fem/operator/common/mapping.hh>
#include <dune/fem/operator/common/operator.hh>
#include <dune/fem/space/common/functionspace.hh>

using namespace Dune;
using namespace Fem;

typedef FunctionSpace<double,double,1,1> SpaceType;
typedef SpaceType :: DomainFieldType DomainFieldType;
typedef SpaceType :: RangeFieldType RangeFieldType;
typedef SpaceType :: DomainType DomainType;
typedef SpaceType :: RangeType  RangeType;

typedef Mapping<
          DomainFieldType,
          RangeFieldType,
          DomainType,
          RangeType > MappingType ;

class A : public Mapping<
          DomainFieldType,
          RangeFieldType,
          DomainType,
          RangeType >
{
  typedef SpaceType :: DomainType DomainType;
  typedef SpaceType :: RangeType  RangeType;

public:
  void apply (const DomainType& arg, RangeType& dest) const
  {
    dest = arg;
    std::cout << "Call A :: apply \n";
  }

};

class B : public Mapping<
          DomainFieldType,
          RangeFieldType,
          DomainType,
          RangeType >
{
  typedef SpaceType :: DomainType DomainType;
  typedef SpaceType :: RangeType  RangeType;

public:
  void apply (const DomainType& arg, RangeType& dest) const
  {
    dest = arg;
    std::cout << "Call B :: apply \n";
  }

};


class C : public Mapping<
          DomainFieldType,
          RangeFieldType,
          DomainType,
          RangeType >
{
  typedef SpaceType :: DomainType DomainType;
  typedef SpaceType :: RangeType  RangeType;

public:
  void operator () (const DomainType& arg, RangeType& dest) const
  {
    dest = arg;
    std::cout << "Call C :: apply \n";
  }

};


int main ()
{
  A a;
  B b;
  C c;

  MappingType m = a + b * 2.0 + a / 2.0 + b - 3.0 * a;
  DomainType arg = 1.0;
  DomainType dest;

  MappingType m1;

  m1( arg, dest );

  m1 = c;
  m1( arg, dest );


  m1 = a + b * 2.0 + c; // result should be 4
  m1( arg, dest );
  std::cout << "Result (should be 4) of m1 combined mapping is: " << dest << std::endl;

  MappingType m2( m1 );
  m1( arg, dest );
  std::cout << "Result (should be 4) of m2 combined mapping is: " << dest << std::endl;

  m(arg,dest);

  std::cout << "Result (should be 1.5) of combined mapping is: " << dest << std::endl;
  MappingType m3 = a - b * 2.0;
  m3(arg,dest);
  std::cout << "Result (should be -1.0) of combined mapping is: " << dest << std::endl;
  return 0;
}
