/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_COLUMNOBJECT_HH
#define DUNE_FEM_COLUMNOBJECT_HH

namespace Dune
{

  namespace Fem
  {

    template< class LinearOperator >
    struct ColumnObject
    {
      typedef typename LinearOperator::ColumnEntityType ColumnEntityType;
      typedef typename LinearOperator::RowEntityType RowEntityType;

      typedef typename LinearOperator::DomainSpaceType DomainSpaceType;
      typedef typename LinearOperator::RangeSpaceType RangeSpaceType;

      typedef typename LinearOperator::LocalMatrixType LocalMatrixType;

      ColumnObject( const LinearOperator &linOp, const ColumnEntityType &colEntity )
      :
        linOp_( linOp ),
        colEntity_( colEntity )
      {}

      //! return local matrix
      inline LocalMatrixType localMatrix( const RowEntityType &rowEntity ) const
      {
        return linOp_.localMatrix( rowEntity, colEntity_ );
      }

      //! return domain space (i.e. space that builds the rows)
      const DomainSpaceType& domainSpace() const { return linOp_.domainSpace(); }

      //! return range space (i.e. space that builds the columns)
      const RangeSpaceType& rangeSpace() const { return linOp_.rangeSpace(); }

    private:
      const LinearOperator &linOp_;
      const ColumnEntityType &colEntity_;
    };

  } // namespace Fem
} // namespace Dune

#endif //#ifndef DUNE_FEM_COLUMNOBJECT_HH
