/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_OPERATOR_MATRIX_FUNCTOR_HH
#define DUNE_FEM_OPERATOR_MATRIX_FUNCTOR_HH

#include <utility>


namespace Dune
{

  namespace Fem
  {

    // IndexFunctor
    // ------------

    template< class Functor, class Index >
    struct IndexFunctor
    {
      constexpr IndexFunctor ( Functor functor, int localIndex, Index globalIndex )
        : functor_( functor ),
          localIndex_( localIndex ),
          globalIndex_( globalIndex )
      {}

      template< class GlobalKey >
      void operator() ( const int localKey, const GlobalKey &globalKey )
      {
        functor_( std::make_pair( localIndex_, localKey ), std::make_pair( globalIndex_, globalKey ) );
      }

    private:
      Functor functor_;
      int localIndex_;
      Index globalIndex_;
    };


    // PairFunctor
    // -----------

    template< class Mapper, class Entity, class Functor >
    struct PairFunctor
    {
      PairFunctor ( const Mapper &mapper, const Entity &entity, Functor functor )
        : mapper_( mapper ), entity_( entity ), functor_( std::move( functor ) )
      {}

      template< class GlobalKey >
      void operator() ( const int localKey, const GlobalKey &globalKey )
      {
        mapper_.mapEach( entity_, IndexFunctor< Functor, GlobalKey >( functor_, localKey, globalKey ) );
      }

    private:
      const Mapper &mapper_;
      const Entity &entity_;
      Functor functor_;
    };


    // makeRowFunctor
    // --------------

    template< class Mapper, class Entity, class Functor >
    PairFunctor< Mapper, Entity, Functor > makePairFunctor ( const Mapper &mapper, const Entity &entity, Functor functor )
    {
      return PairFunctor< Mapper, Entity, Functor >( mapper, entity, std::move( functor ) );
    }

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_OPERATOR_MATRIX_FUNCTOR_HH
