/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_OPERATOR_PROJECTION_LOCAL_RIESZ_LOCALRIESZPROJECTION_HH
#define DUNE_FEM_OPERATOR_PROJECTION_LOCAL_RIESZ_LOCALRIESZPROJECTION_HH

#include <dune/common/bartonnackmanifcheck.hh>

namespace Dune
{

  namespace Fem
  {

    // LocalRieszProjection
    // --------------------

    /** \class LocalRieszProjection
     *
     *  \brief interface documentation of a local Riesz projection
     *
     *  \tparam  BasisFunctionSet  basis function set type
     */
    template< class BasisFunctionSet, class Implementation >
    class LocalRieszProjection
    {
    public:
      /** \brief basis function set */
      typedef BasisFunctionSet BasisFunctionSetType;

    protected:
#ifndef DOXYGEN

      LocalRieszProjection () = default;

#endif // #ifndef DOXYGEN

      /** \name Copying and assignment
       *  \{
       */

      /** \brief copy constructor */
      LocalRieszProjection ( const LocalRieszProjection & ) = default;

      /** \brief move constructor */
      LocalRieszProjection ( LocalRieszProjection && ) {}

      /** \brief assignment operator */
      LocalRieszProjection &operator= ( const LocalRieszProjection & ) = default;

      /** \brief move assignment operator */
      LocalRieszProjection &operator= ( LocalRieszProjection && ) {}

      /** \} */

      /** \name Public member methods
       *  \{
       */

      /** \brief return basis function set */
      BasisFunctionSet basisFunctionSet () const
      {
        CHECK_INTERFACE_IMPLEMENTATION( impl().basisFunctionSet() );
        return impl().basisFunctionSet();
      }

      /** \brief compute Riesz representation
       *
       *  \tparam  LocalDofVector  local dof vector type
       *
       *  \param[in]  f  please doc me
       *  \param[out]  localDofVector  please doc me
       */
      template< class F, class LocalDofVector >
      void operator() ( const F &f, LocalDofVector &localDofVector ) const
      {
        impl()( f, localDofVector );
      }

      /** \brief compute Riesz representation
       *
       *  \tparam  LocalDofVector  local dof vector type
       *
       *  \param[in]  f  please doc me
       *  \param[out]  localDofVector  please doc me
       */
      template< class F, class LocalDofVector >
      void apply ( const F &f, LocalDofVector &localDofVector ) const
      {
        CHECK_INTERFACE_IMPLEMENTATION( impl().apply( f, localDofVector ) );
        impl().apply( f, localDofVector );
      }

      /** \} */

    protected:
      const Implementation &impl () const
      {
        return static_cast< const Implementation & >( *this );
      }
    };

  } // namespace Fem

} // namepsace Dune

#endif // #ifndef DUNE_FEM_OPERATOR_PROJECTION_LOCAL_RIESZ_LOCALRIESZPROJECTION_HH
