/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_POINTMAPPER_HH
#define DUNE_FEM_POINTMAPPER_HH

//- system includes
#include <vector>

//- Dune includes
#include <dune/common/version.hh>
#include <dune/fem/quadrature/quadrature.hh>

namespace Dune
{

  namespace Fem
  {

    struct QuadratureKey
    {
      QuadratureKey ( const GeometryType &geoType, const size_t id )
      : id_( ((topologyId( geoType ) >> 1) << 16) + id )
      {
        assert( id < (1 << 16) );
      }

      bool operator< ( const QuadratureKey &other ) const
      {
        return (id_ < other.id_);
      }

      bool operator== ( const QuadratureKey &other ) const
      {
        return (id_ == other.id_);
      }

      friend std::ostream &operator<< ( std::ostream &out, const QuadratureKey &key )
      {
        return out << "(topologyId " << ((key.id_ >> 16) << 1) << ", quadId " << (key.id_ & ((1u << 16)-1)) << ")";
      }

    protected:
      static unsigned int topologyId ( const GeometryType &type )
      {
        return type.id();
      }

      const size_t id_;
    };



    template <class ct, int dim>
    struct CachingTraits {
      //! type of integration point list implementation, fix type here
      typedef IntegrationPointListImp<ct, dim> QuadratureType;
      //! extracted types from integration point list
      typedef typename QuadratureType::CoordinateType PointType;
      typedef std::vector<PointType>    PointVectorType;
      typedef std::vector<size_t>       MapperType;
      typedef std::vector<MapperType>   MapperVectorType;

      typedef QuadratureKey QuadratureKeyType;

      // minimal twist is -4 for hexahedrons
      // so we add 4 to start from zero
      enum { twistOffset_ = 4 };

    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_POINTMAPPER_HH
