/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_REFELEM_TEST_HH
#define DUNE_REFELEM_TEST_HH

#include <dune/geometry/type.hh>
#include <dune/common/fvector.hh>


namespace Dune
{
  namespace Fem
  {

  class ReferenceElement_Test
  {
    GeometryType cube3_;
    GeometryType cube2_;
    GeometryType simplex3_;
    GeometryType simplex2_;
    GeometryType simplex1_;

  public:
    ReferenceElement_Test ()
#if 0
    : cube3_(GeometryType(GeometryType::cube,3)),
      cube2_(GeometryType(GeometryType::cube,2)),
      simplex3_(GeometryType(GeometryType::simplex,3)),
      simplex2_(GeometryType(GeometryType::simplex,2)),
      simplex1_(GeometryType(GeometryType::simplex,1)),
      refCube3_(ReferenceElements<double, 3>::general(cube3_)),
      refSimplex3_(ReferenceElements<double, 3>::general(simplex3_)),
      refCube2_(ReferenceElements<double, 2>::general(cube2_)),
      refSimplex2_(ReferenceElements<double, 2>::general(simplex2_)),
      refLine_(ReferenceElements<double, 1>::general(simplex1_))
#endif
    {}

    virtual void run();

#if 0
    void globalTest();
    void allGeometriesTest();

  private:
    template <int dim, int codim>
    void checkCorners(const ReferenceElement<double, dim>& refElem);

    template <typename RefElemType, int codim>
    void checkSingle(const RefElemType& ref);

  private:
    const ReferenceElement<double, 3>& refCube3_;
    const ReferenceElement<double, 3>& refSimplex3_;
    const ReferenceElement<double, 2>& refCube2_;
    const ReferenceElement<double, 2>& refSimplex2_;
    const ReferenceElement<double, 1>& refLine_;
#endif
  };

  } // end namespace Fem
} // end namespace Dune

#endif
