/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#include "topology.hh"

namespace Dune
{
  namespace Fem
  {
    //- class FaceTopologyMapping
    template <>
    int FaceTopologyMapping<tetra>::
    twist(int index, int faceTwist) {
      return (faceTwist < 0) ?
        (7 - index + faceTwist)%3 : (faceTwist + index)%3 ;
    }

    template <>
    int FaceTopologyMapping<hexa>::
    twist(int index, int faceTwist) {
      return (faceTwist < 0) ?
        (9 - index + faceTwist)%4 : (faceTwist + index)%4 ;
    }

    template <>
    int FaceTopologyMapping<tetra>::
    invTwist(int index, int faceTwist)
    {
      return (faceTwist < 0) ?
        (7 - index + faceTwist)%3 : (3 + index - faceTwist)%3;
    }

    template <>
    int FaceTopologyMapping<hexa>::
    invTwist(int index, int faceTwist) {
      return (faceTwist < 0) ?
        (9 - index + faceTwist)%4 : (4 + index - faceTwist)%4;
    }

    // alu triangle face are oriented just the other way then dune faces
    // therefore vertex 1 and 2 are swapped because
    // ALUGrid tetra face are oriented just the other way compared to Dune
    // tetra faces, see also gitter_geo.cc of the ALUGrid code
    template <>
    const int FaceTopologyMapping<tetra>::
    dune2aluVertex_[EntityCount<tetra>::numVerticesPerFace] = {0, 2, 1};

    // mapping of twists from alu 2 dune
    template <>
    const int FaceTopologyMapping<tetra>::
    alu2duneTwist_[ 2 * EntityCount<tetra>::numVerticesPerFace] = { -2, -3, -1, 0, 2, 1 };

    // mapping of twists from alu 2 dune
    template <>
    const int FaceTopologyMapping<tetra>::
    aluTwistMap_[ 2 * EntityCount<tetra>::numVerticesPerFace] = { 1, 2, 0, -1, -2, -3 };

    // mapping of twists from alu 2 dune
    template <>
    const int FaceTopologyMapping<hexa>::
    aluTwistMap_[ 2 * EntityCount<hexa>::numVerticesPerFace] = { -2, -3, -4, -1, 0, 1, 2, 3 } ;

    // the mapping of vertices in the reference quad
    // this is used for hexa face during intersection iterator build
    // and to calculate the intersectionSelfLocal and
    // intersectionSelfNeighbor geometries.
    template <>
    const int FaceTopologyMapping<hexa>::
    dune2aluVertex_[EntityCount<hexa>::numVerticesPerFace] = {0, 3, 1, 2};

    // alu triangle face are oriented just the other way then dune faces
    // therefore vertex 1 and 2 are swaped
    template <>
    const int FaceTopologyMapping<tetra>::
    alu2duneVertex_[EntityCount<tetra>::numVerticesPerFace] = {0, 2, 1};

    template <>
    const int FaceTopologyMapping<hexa>::
    alu2duneVertex_[EntityCount<hexa>::numVerticesPerFace] = {0, 2, 3, 1};

    template <>
    const int FaceTopologyMapping<tetra>::
    dune2aluEdge_[EntityCount<tetra>::numEdgesPerFace] = {1, 2, 0};

    template <>
    const int FaceTopologyMapping<hexa>::
    dune2aluEdge_[EntityCount<hexa>::numEdgesPerFace] = {0, 2, 3, 1};

    template <>
    const int FaceTopologyMapping<tetra>::
    alu2duneEdge_[EntityCount<tetra>::numEdgesPerFace] = {2, 0, 1};

    template <>
    const int FaceTopologyMapping<hexa>::
    alu2duneEdge_[EntityCount<hexa>::numEdgesPerFace] = {0, 3, 1, 2};

  } // end namespace Fem
} // end namespace Dune
