/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_SPACE_PADAPTIVE_ADAPTMANAGER_HH
#define DUNE_FEM_SPACE_PADAPTIVE_ADAPTMANAGER_HH

#include <dune/grid/common/capabilities.hh>

#include <dune/fem/function/adaptivefunction.hh>
#include <dune/fem/operator/lagrangeinterpolation.hh>
#include <dune/fem/space/common/localrestrictprolong.hh>
#include <dune/fem/space/discontinuousgalerkin/localrestrictprolong.hh>

#include "declaration.hh"
#include "restrictprolong.hh"


namespace Dune
{

  namespace Fem
  {

    // DefaultLocalRestrictProlong
    // ---------------------------

    template< class FS, class GP, int ord, template< class > class S >
    class DefaultLocalRestrictProlong< Fem::PAdaptiveLagrangeSpace< FS, GP, ord, S > >
    : public PLagrangeLocalRestrictProlong< typename GP::GridType, Fem::PAdaptiveLagrangeSpace< FS, GP, ord, S > >
    {
    public:
      DefaultLocalRestrictProlong ( const Fem::PAdaptiveLagrangeSpace< FS, GP, ord, S > &space )
      : PLagrangeLocalRestrictProlong< typename GP::GridType, Fem::PAdaptiveLagrangeSpace< FS, GP, ord, S > >( space )
      {}
    };


    template< class FunctionSpaceImp, class GridPartImp, int polOrd, template< class > class StorageImp >
    class DefaultLocalRestrictProlong< Fem::PAdaptiveDGSpace< FunctionSpaceImp, GridPartImp, polOrd, StorageImp > >
    : public DiscontinuousGalerkinLocalRestrictProlong< Fem::PAdaptiveDGSpace< FunctionSpaceImp, GridPartImp, polOrd, StorageImp >, false > // invert mass matrix or not
    {
    public:
      typedef DiscontinuousGalerkinLocalRestrictProlong< Fem::PAdaptiveDGSpace<
                                                         FunctionSpaceImp,
                                                         GridPartImp,
                                                         polOrd, StorageImp >, false > BaseType ;
      DefaultLocalRestrictProlong ( const Fem::PAdaptiveDGSpace< FunctionSpaceImp, GridPartImp, polOrd, StorageImp > & space )
        : BaseType( space )
      {}
    };

    template< class FunctionSpaceImp, class GridPartImp, template< class > class StorageImp >
    class DefaultLocalRestrictProlong< Fem::PAdaptiveDGSpace< FunctionSpaceImp, GridPartImp, 0, StorageImp > >
    : public ConstantLocalRestrictProlong< Fem::PAdaptiveDGSpace< FunctionSpaceImp, GridPartImp, 0, StorageImp > >
    {
    public:
      DefaultLocalRestrictProlong ( const Fem::PAdaptiveDGSpace< FunctionSpaceImp, GridPartImp, 0, StorageImp > & )
      {}
    };


    // pAdaptation
    // -----------

    template <class DF, class Vector, class DFS>
    void pAdaptation( DF& df, const Vector& polynomialOrders, const DFS &space, const int )
    {}

    /** \brief pAdaptation
        \param df  discrete function to adapt
        \param polynomialOrders  vector containing polynomial orders for each cell
        \param space  type of space tp be adapted
        \param polOrderShift possible shift of polynomial order (i.e. in case of
                             Taylor-Hood put -1 for the pressure) (default = 0)
    */
    template <class DF, class Vector,
              class FS, class GP, int p,
              template< class > class Storage >
    void pAdaptation( DF& df,
                      const Vector& polynomialOrders,
                      const Fem::PAdaptiveLagrangeSpace<FS,GP,p,Storage> &space,
                      const int polOrderShift = 0 )
    {
      /*
      typedef typename DF :: DiscreteFunctionSpaceType  DiscreteFunctionSpaceType;
      typedef typename DiscreteFunctionSpaceType :: GridPartType GridPartType;
      typedef typename GridPartType :: GridType  GridType;

      DiscreteFunctionSpaceType& newSpace = const_cast< DiscreteFunctionSpaceType& > (df.space());
      DiscreteFunctionSpaceType oldSpace( df.space().gridPart() );

      typedef DofManager< GridType > DofManagerType;
      DofManagerType& dm = DofManagerType :: instance( newSpace.grid() );

      for( const auto& entity : newSpace )
        oldSpace.blockMapper().setPolynomOrder( entity, newSpace.blockMapper().polynomOrder( entity ) );

      dm.resize();
      dm.compress();

      AdaptiveDiscreteFunction< DiscreteFunctionSpaceType > tmp( "padaptation", oldSpace );

      newSpace.adapt( tmp )
      tmp.assign( df );

      for( const auto& entity : newSpace )
      {
        const int polOrder = polynomialOrders[ newSpace.indexSet().index( entity ) ] + polOrderShift ;
        newSpace.blockMapper().setPolynomOrder( entity, polOrder );
      }

      dm.resize();
      dm.compress();

      LagrangeInterpolation< DF > :: interpolateFunction( tmp, df );
      */
    }

    /** \brief pAdaptation
        \param df  discrete function to adapt
        \param polynomialOrders  vector containing polynomial orders for each cell
        \param polOrderShift possible shift of polynomial order (i.e. in case of
          Taylor-Hood put -1 for the pressure) (default = 0)
    */
    template <class DF, class Vector>
    void pAdaptation( DF& df,
                      const Vector& polynomialOrders,
                      const int polOrderShift = 0 )
    {
      pAdaptation( df, polynomialOrders, df.space(), polOrderShift );
    }

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_SPACE_PADAPTIVE_ADAPTMANAGER_HH
