/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_SPACE_SHAPEFUNCTIONSET_LEGENDREPOLYNOMIALS_HH
#define DUNE_FEM_SPACE_SHAPEFUNCTIONSET_LEGENDREPOLYNOMIALS_HH

// C++ inlcudes
#include <cassert>
#include <iostream>


namespace Dune
{

  namespace Fem
  {

    class LegendrePolynomials
    {
    public:
      static const int maxOrder = 11;

      static const double factor[ maxOrder ][ maxOrder ];
      static const double weight[ maxOrder ];

    public:
      static double evaluate ( const int num, const double x )
      {
        assert( 0 <= num && num < maxOrder );

        double phi = factor[ num ][ num ];
        for( int i = num-1; i >= 0; --i )
          phi = phi * x + factor[ num ][ i ];
        return weight[ num ] * phi;
      }

      static double jacobian ( const int num, const double x )
      {
        assert( 0 <= num && num < maxOrder );

        double phi = 0.;
        if( num >= 1 )
        {
          phi = factor[ num ][ num ] * num;
          for( int i = num-1; i >= 1; --i )
            phi = phi * x + factor[ num ][ i ] * i;
        }
        return weight[ num ] * phi;
      }

      static double hessian ( const int num, const double x )
      {
        assert( 0 <= num && num < maxOrder );

        double phi=0.;
        if( num >= 2 )
        {
          phi = factor[ num ][ num ] * num * ( num-1 );
          for( int i = num-1; i >= 2; --i )
            phi = phi * x + factor[ num ][ i ] * i * (i-1);
        }
        return weight[ num ] * phi;
      }
    };

  } // namespace Fem

} // namespace Dune

#endif // #ifndef DUNE_FEM_SPACE_SHAPEFUNCTIONSET_LEGENDREPOLYNOMIALS_HH
