/**************************************************************************
 
  The dune-fem module is a module of DUNE (see www.dune-project.org).
  It is based on the dune-grid interface library 
  extending the grid interface by a number of discretization algorithms
  for solving non-linear systems of partial differential equations.

  Copyright (C) 2003 - 2015 Robert Kloefkorn
  Copyright (C) 2003 - 2010 Mario Ohlberger 
  Copyright (C) 2004 - 2015 Andreas Dedner
  Copyright (C) 2005        Adrian Burri
  Copyright (C) 2005 - 2015 Mirko Kraenkel
  Copyright (C) 2006 - 2015 Christoph Gersbacher
  Copyright (C) 2006 - 2015 Martin Nolte
  Copyright (C) 2011 - 2015 Tobias Malkmus
  Copyright (C) 2012 - 2015 Stefan Girke
  Copyright (C) 2013 - 2015 Claus-Justus Heine
  Copyright (C) 2013 - 2014 Janick Gerstenberger
  Copyright (C) 2013        Sven Kaulman
  Copyright (C) 2013        Tom Ranner
  Copyright (C) 2015        Marco Agnese
  Copyright (C) 2015        Martin Alkaemper


  The dune-fem module is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.

  The dune-fem module is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along
  with this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
**************************************************************************/
#ifndef DUNE_FEM_TEST_WEIGHTFUNCTION_HH
#define DUNE_FEM_TEST_WEIGHTFUNCTION_HH

#include <dune/fem/function/common/function.hh>

namespace Dune
{

  template< class FunctionSpaceImp >
  class WeightFunction
  : public Fem::Function< FunctionSpaceImp, WeightFunction< FunctionSpaceImp > >
  {
    typedef WeightFunction< FunctionSpaceImp > ThisType;
    typedef Fem::Function< FunctionSpaceImp, ThisType > BaseType;

  public:
    typedef FunctionSpaceImp FunctionSpaceType;

    typedef typename FunctionSpaceType::DomainFieldType DomainFieldType;
    typedef typename FunctionSpaceType::RangeFieldType RangeFieldType;

    typedef typename FunctionSpaceType::DomainType DomainType;
    typedef typename FunctionSpaceType::RangeType RangeType;
    typedef typename FunctionSpaceType::JacobianRangeType JacobianRangeType;

  public:
    void evaluate( const DomainType &x, RangeType &phi ) const
    {
      phi = 1;
      if( x[0] > 0.5 )
        phi *= 0.5;
    }

    void evaluate( const DomainType &x, RangeFieldType t, RangeType &phi ) const
    {
      evaluate( x, phi );
    }
  };

}

#endif // #ifndef DUNE_FEM_TEST_WEIGHTFUNCTION_HH
